from .command import CommandQuery
from .delegateion_query import DelegationQuery
from .illust import IllustQuery
from .more import MoreQuery
from .query import Query, register_query
from .random_bookmark import RandomBookmarkQuery
from .random_illust import RandomIllustQuery
from .random_recommended_illust import RandomRecommendedIllustQuery
from .random_related_illust import RandomRelatedIllustQuery
from .random_user_illust import RandomUserIllustQuery
from .ranking import RankingQuery

__all__ = ("CommandQuery", "DelegationQuery", "IllustQuery", "MoreQuery", "Query", "RandomBookmarkQuery", "RandomIllustQuery",
           "RandomRecommendedIllustQuery", "RandomRelatedIllustQuery", "RandomUserIllustQuery", "RankingQuery",
           "register_query")
